/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: EarDeployableMetadataFactory.java 4610 2009-02-17 08:49:37Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ear.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.ee.deploy.api.deployable.CARDeployable;
import org.ow2.util.ee.deploy.api.deployable.EARDeployable;
import org.ow2.util.ee.deploy.api.deployable.EJB3Deployable;
import org.ow2.util.ee.deploy.api.deployable.LibDeployable;
import org.ow2.util.ee.deploy.api.deployable.WARDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.car.impl.CarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ear.api.IEarDeployableMetadata;
import org.ow2.util.ee.metadata.ear.api.IEarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.impl.internal.DefaultEjbJarDeployableFactory;
import org.ow2.util.ee.metadata.war.api.IWarDeployableMetadata;
import org.ow2.util.ee.metadata.war.impl.WarDeployableMetadataFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;
import org.ow2.util.scan.impl.ASMScannerImpl;
import org.ow2.util.url.URLUtils;
import org.ow2.util.url.URLUtilsException;

/**
 * Implementation of ear deployable metadata.
 * @author Gael Lalire
 */
public class EarDeployableMetadataFactory implements IEarDeployableMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(EarDeployableMetadataFactory.class);

    /**
     * The factory which manage war.
     */
    private WarDeployableMetadataFactory warDeployableMetadataFactory;

    /**
     * The factory which manage client archive.
     */
    private CarDeployableMetadataFactory carDeployableMetadataFactory;

    /**
     * The factory which manage ejbjar.
     */
    private EjbJarDeployableMetadataFactory<EJB3Deployable, ?, ?, ?, ?> ejbJarDeployableMetadataFactory;

    /**
     * Constructor. Use {@link ASMScannerImpl} to analyze class bytecode.
     */
    public EarDeployableMetadataFactory() {
        this(new ASMScannerImpl());
    }

    /**
     * Constructor.
     * @param scanner the scanner which analyze class bytecode.
     */
    @SuppressWarnings("unchecked")
    public EarDeployableMetadataFactory(final IScanner scanner) {
        warDeployableMetadataFactory = new WarDeployableMetadataFactory(scanner);
        carDeployableMetadataFactory = new CarDeployableMetadataFactory(scanner);
        ejbJarDeployableMetadataFactory = new DefaultEjbJarDeployableFactory<EJB3Deployable>(scanner);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @return a new ear deployable metadata
     * @throws DeployableMetadataException if ear is not uncompressed
     */
    public IEarDeployableMetadata createDeployableMetadata(final EARDeployable deployable) throws DeployableMetadataException {
        return createDeployableMetadata(deployable, null);
    }

    /**
     * @param deployable the ear deployable (getOriginalDeployable must not be
     *        null ie it's already decompressed archive)
     * @param classLoader a classloader to use if a class is not in archive
     * @return a new ear deployable metadata
     * @throws DeployableMetadataException if ear is not uncompressed
     */
    public IEarDeployableMetadata createDeployableMetadata(final EARDeployable deployable, final ClassLoader classLoader)
            throws DeployableMetadataException {

        long tAnalyzeStart = System.currentTimeMillis();

        try {
            if (!URLUtils.urlToFile2(deployable.getArchive().getURL()).isDirectory()) {
                throw new DeployableMetadataException("ear must be uncompressed (local directory expected)");
            }
        } catch (URLUtilsException e) {
            throw new DeployableMetadataException("ear must be uncompressed (local directory expected)");
        } catch (ArchiveException e) {
            throw new DeployableMetadataException("ear exception", e);
        }
        List<LibDeployable> libDeployableList = deployable.getLibDeployables();
        List<IClassesLocator> libraryList = new ArrayList<IClassesLocator>();
        for (LibDeployable libDeployable : libDeployableList) {
            libraryList.add(new ArchiveClassesLocator(libDeployable.getArchive()));
        }
        if (classLoader != null) {
            libraryList.add(new ClassLoaderClassesLocator(classLoader));
        } else {
            libraryList.add(new ClassLoaderClassesLocator(ClassLoader.getSystemClassLoader()));
        }
        EarDeployableMetadata earDeployableMetadata = new EarDeployableMetadata(deployable);

        // war
        for (WARDeployable warDeployable : deployable.getWARDeployables()) {
            IWarDeployableMetadata warDeployableMetadata = warDeployableMetadataFactory.createDeployableMetadata(warDeployable,
                    libraryList, null);
            earDeployableMetadata.addWarDeployableMetadata(warDeployableMetadata);
        }

        // car
        for (CARDeployable carDeployable : deployable.getCARDeployables()) {
            ICarDeployableMetadata carDeployableMetadata = carDeployableMetadataFactory.createDeployableMetadata(carDeployable,
                    libraryList, null);
            earDeployableMetadata.addCarDeployableMetadata(carDeployableMetadata);
        }

        // ejb jar
        for (EJB3Deployable ejbJarDeployable : deployable.getEJB3Deployables()) {
            IEjbJarDeployableMetadata<EJB3Deployable, ?, ?, ?, ?> ejbJarDeployableMetadata = ejbJarDeployableMetadataFactory
                    .createDeployableMetadata(ejbJarDeployable, libraryList);
            earDeployableMetadata.addEjbJarDeployableMetadata(ejbJarDeployableMetadata);
        }

        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            LOGGER.debug("Analyze of file {0} took {1} ms.", deployable.getArchive().getName(),
                    (tAnalyzeStartEnd - tAnalyzeStart));
        }

        return earDeployableMetadata;
    }

    /**
     * @return the supported class of this factory
     */
    public Class<IEarDeployableMetadata> getDeployableMetadataClass() {
        return IEarDeployableMetadata.class;
    }

}
